
# LECA Bash library
#
# The LECA bash library provides a set of function used for helping
# development of bash script mainly to write job script on the luke
# cluster
#
# The files from the LECA Bash library must be sourced from your main script
#
#

include logging
logdebug "Load mutex package"

if [[ -z "$LOCK_DIR" ]] ; then
  LOCK_DIR="${HOME}/lecalock"
fi

# Create the lockdir if it does not exist
if [[ ! -d "${LOCK_DIR}" ]]; then
	mkdir -p "${LOCK_DIR}"
	chmod a+rwxt "${LOCK_DIR}"
fi

include clobber

LECA_LOCK_DELAY=2
LECA_LOCK_MAXRETRY=0

function __lockfile__() {
  echo "${LOCK_DIR}/__${1}__.lecalock"
}

function __locktag__() {
  echo "$(hostname).$$"
}


function trytogetlock() {
  local lname="$1"
  shift

  local mode="user"

  logdebug "Try to get lock ${lname}"
  
  if [[ "${1}" == "global" ]]; then
    mode="global"
  fi

  if [[ "${mode}" == "global" ]]; then
    name=$(__lockfile__ "${lname}")
  else
    name=$(__lockfile__ "${lname}.$(whoami)")
  fi

  local TAG=$(__locktag__)
  local retry=0

  local notgetlock=1

  pushnoclobber
    echo "${TAG}" 2>/dev/null 1> "${name}"
    notgetlock=$?
  popnoclobber
  
  if (( notgetlock > 0 )); then
    if [[ $(cat "${name}" 2> /dev/null) == "${TAG}" ]]; then
      notgetlock=0
    fi
  fi

  if (( notgetlock == 0 )); then
	  logdebug "Succeed to get lock ${lname}"
  fi

  if (( notgetlock > 0 )); then
    return $notgetlock
  else
  	chmod a+w "${name}"
    return 0
  fi
}

function getlock() {
  local lname="$1"
  shift

  local mode="user"
  
  logdebug "Try to get lock ${lname}"

  if [[ "${1}" == "global" ]]; then
    mode="global"
  fi

  if [[ "${mode}" == "global" ]]; then
    name=$(__lockfile__ "${lname}")
  else
    name=$(__lockfile__ "${lname}.$(whoami)")
  fi

  local TAG=$(__locktag__)
  local retry=0

  local notgetlock=1

  while (( notgetlock > 0 && ( LECA_LOCK_MAXRETRY == 0 || retry > LECA_LOCK_MAXRETRY) )); do

    pushnoclobber
      echo "${TAG}" 2>/dev/null 1> "${name}"
      notgetlock=$?
    popnoclobber

    if (( notgetlock > 0 )); then
      if [[ $(cat "${name}" 2> /dev/null) == "${TAG}" ]]; then
        notgetlock=0
      fi
    fi

    if (( notgetlock > 0 )); then
      logdebug "Failed to get lock ${lname}"
      sleep "${LECA_LOCK_DELAY}"
      retry=$((retry + 1))
    fi

  done

  if (( notgetlock > 0 )); then
    return $notgetlock
  else
	logdebug "Succeed to get lock ${lname}"
  	chmod a+w "${name}"
    return 0
  fi
}

function releaselock() {
  local lname="$1"
  shift

  local mode="user"
  
  logdebug "Try to release lock ${lname}"
  
  if [[ "${1}" == "global" ]]; then
    mode="global"
  fi

  if [[ "${mode}" == "global" ]]; then
    name=$(__lockfile__ "${lname}")
  else
    name=$(__lockfile__ "${lname}.$(whoami)")
  fi

  local TAG=$(__locktag__)
  local LOCK=""

  if [[ -f "${name}" ]]; then
    LOCK=$(cat "${name}" 2> /dev/null)
  else
    return 0
  fi


  if [[ "${LOCK}" == "${TAG}" ]]; then
    rm -f "${name}"
    logdebug "Lock ${lname} released"
    return 0
  fi

  return 1
}


function getsemaphore() {
  local semname="$1"
  shift

  local maxcount="$1"
  shift
  
  local name=""
  local mode="user"
  
  if [[ "${1}" == "global" ]]; then
    mode="global"
  fi

  local TAG=$(__locktag__)
  local retry=0
  local notgetlock=1

  while (( notgetlock > 0 && ( LECA_LOCK_MAXRETRY == 0 || retry > LECA_LOCK_MAXRETRY) )); do
	  local i=0
	  
	  while (( notgetlock > 0 )) && (( i < maxcount )) ; do
	  
		if [[ "${mode}" == "global" ]]; then
		  name=$(__lockfile__ "${semname}.@${i}")
		else
		  name=$(__lockfile__ "${semname}.$(whoami).@${i}")
		fi
			  
		pushnoclobber
		  echo "${TAG}" 2>/dev/null 1> "${name}"
		  notgetlock=$?
		popnoclobber
		
        if (( notgetlock > 0 )); then
	        i=$((i + 1))
        fi
        
	  done
  	
      if (( notgetlock > 0 )); then
        loginfo "Failed to get semaphore $semname"
        sleep "${LECA_LOCK_DELAY}"
        retry=$((retry + 1))
      fi
  done

  if (( notgetlock > 0 )); then
    return $notgetlock
  else
  	chmod a+w "${name}"
    return 0
  fi
  
}

function releasesemaphore() {
  local name="$1"
  shift

  local mode="user"
  
  if [[ "${1}" == "global" ]]; then
    mode="global"
  fi

  if [[ "${mode}" == "global" ]]; then
	  name=$(__lockfile__ "${name}.@*")
  else
	  name=$(__lockfile__ "${name}.$(whoami).@*")
  fi

  toremove=$(ls -tr $name  2> /dev/null | head -1)
  
  if [[ ! -z "$toremove" ]] ; then
      rm -f "$toremove"
      return 0
  fi
  
  return 1
 }
 
 
 function releaseoldlock() {
	local minutes=10
	
	if [[ ! -z "$1" ]]; then
		minutes="$1"
	fi
	
	for lock in $(find "${LOCK_DIR}" -type f -cmin "+${minutes}" -print); do
		rm "${lock}"
		loginfo "releasing old lock: ${lock}"
	done

}
 